Attribute VB_Name = "Main"
Option Explicit

Type Options
    SearchFileName As String
    SearchWindowSize As String
    SearchStartPosition As Long     'Used for .INI and CommandLine
    DisplayGUI As Integer           'Used for .INI and CommandLine
    AdjacentDigits As Integer       'The number of adjacent digits to return on left and right of search string
    BCDMode As Integer              'Specifies BCD file format for numeric search
End Type

Private Type CommandOptions
    SearchString As String
    OutputFile As String
    CommandLineError As Boolean
End Type

Private Type Result
    TextPosition As String
    LeftDigits As String
    RightDigits As String
    SearchTime As String
    FileSize As Long
    ErrorCode As Long
    StatusFlag As Boolean
End Type

Private Type Process
    StartTime As Single
    ElapsedTime As Single
End Type

Public Options As Options
Dim CommandOptions As CommandOptions
Dim Result As Result
Dim Process As Process

'Module Level Errors
Private Enum ERROR
    BASE = 65000
    INVALID_COMMAND_LINE = 65000 + 1
    SEARCH_FILE_NOT_SPECIFIED = 65000 + 2
    SEARCH_FILE_INVALID = 65000 + 3
    COMMAND_LINE_OOB = 65000 + 4
End Enum



Public Sub WriteOutputFile()
'---------------------------------------------------------------------------
'WriteOutputFile generates results to file specified in
'   CommandOptions.OutputFile.
'
'   Process.ElapsedTime represents to the total processing time, including
'   overhead, from the time the program is executed.
'
'---------------------------------------------------------------------------
On Error Resume Next

Const TEXT_POSITION = "TextPosition="
Const SEARCH_TIME = "SearchTime="
Const LEFT_DIGITS = "LeftDigits="
Const RIGHT_DIGITS = "RightDigits="
Const ERROR_CODE = "ErrorCode="
Const SEARCH_STRING = "SearchString="
Const FILE_SIZE = "FileSize="
Const SEARCH_START_POSITION = "SearchStartPosition="
    
    If PathExistFull(CommandOptions.OutputFile) Then
        Open CommandOptions.OutputFile For Output As #1
            Print #1, TEXT_POSITION & Result.TextPosition
            Print #1, SEARCH_TIME & Process.ElapsedTime
            Print #1, LEFT_DIGITS & Result.LeftDigits
            Print #1, RIGHT_DIGITS & Result.RightDigits
            Print #1, ERROR_CODE & Result.ErrorCode
            Print #1, SEARCH_STRING & CommandOptions.SearchString
            Print #1, FILE_SIZE & Result.FileSize
            Print #1, SEARCH_START_POSITION & Options.SearchStartPosition
        Close #1
    End If
    
End Sub

Public Sub CommandLine(ByVal Command As String)
'---------------------------------------------------------------------------
'{CommandLine} process command line arguments passed to the app and assigns
'   the appropriate values to the CommandOptions type
'
'   CmdLine is Command$ as passed from the startup module (frmTextPos)
'
'---------------------------------------------------------------------------
On Error GoTo ErrorHandle

Dim ItemPos As Integer, CmdItem As String, CmdItemVal As String

Const SEARCH_STRING As String = "s" 'Valid Command Line Switches
Const OUTPUT_FILE As String = "o"
Const SEARCH_START_POSITION = "p"
Const ADJACENT_DIGITS = "a"
Const DISPLAY_GUI As String = "g"
Const BCD_MODE As String = "b"
Const OPT_DELIM = "/", VAL_DELIM = "="

    CommandOptions.CommandLineError = False 'Initialize default state
    
    If Len(Command) < 1 Then
        Exit Sub
    End If
    
    For ItemPos = 2 To ItemCount(Command, OPT_DELIM)
        CmdItem = LCase(ParseEndSpace(ItemParse(Command, ItemPos, OPT_DELIM)))
        CmdItemVal = ItemParse(CmdItem, 2, VAL_DELIM)
        Select Case Left(CmdItem, 1)
            Case SEARCH_STRING
                CommandOptions.SearchString = CmdItemVal
            Case OUTPUT_FILE
                CommandOptions.OutputFile = CmdItemVal
            Case SEARCH_START_POSITION
                Options.SearchStartPosition = CDbl(Val(CmdItemVal))
                If Options.SearchStartPosition < 1 Then Options.SearchStartPosition = 1
            Case ADJACENT_DIGITS
                Options.AdjacentDigits = CInt(Abs(255 And (Val(CmdItemVal) Or Val(CmdItemVal) > 255)))
            Case DISPLAY_GUI
                Options.DisplayGUI = 1 And CInt(CmdItemVal)
            Case BCD_MODE
                Options.BCDMode = 1 And CInt(CmdItemVal)
            Case Else
                CommandOptions.CommandLineError = True
        End Select
    Next
    
    Exit Sub

ErrorHandle:
    Result.ErrorCode = ERROR.COMMAND_LINE_OOB
    If Options.DisplayGUI = 0 Then
        WriteOutputFile
        End
    End If
    Resume Next
    
End Sub

Public Sub InitializeApp(ByVal Command As String)
'---------------------------------------------------------------------------
'InitializeApp scripts initialization functionality and runs command-line
'   search if a SearchString is specified on the command-line.
'
'   [Command] is passed through from the startup module (frmTextPos) in
'   order to pass it along again to the CommandLine(..) SubRoutine
'
'---------------------------------------------------------------------------

    Process.StartTime = Timer
    
    Settings (1)                'Load Settings from .INI file
    CommandLine (Command)       'Process Command Line options - overides .INI settings
    Result.TextPosition = "0"   'Default value

    If Not Options.DisplayGUI = 1 Then
        If ValidateCommandOptions And Not CommandOptions.CommandLineError Then
            If Options.SearchFileName <> "" Then
                If FileExist(Options.SearchFileName) Then 'All OK, Execute and output
                    RunSearch (CommandOptions.SearchString)
                    Process.ElapsedTime = Abs(Timer - Process.StartTime)
                    WriteOutputFile
                    End
                  Else 'Terminate if invalid SerarchFileName
                    Result.ErrorCode = ERROR.SEARCH_FILE_INVALID
                    WriteOutputFile
                    End
                End If
              Else
                Result.ErrorCode = ERROR.SEARCH_FILE_NOT_SPECIFIED
                WriteOutputFile
                End
            End If
          Else
            Result.ErrorCode = ERROR.INVALID_COMMAND_LINE
            WriteOutputFile
            End
        End If
    End If
    
End Sub

Public Function ValidateCommandOptions() As Boolean
'---------------------------------------------------------------------------
'This routine validates commandline input for automated search and sets
'   default a sets a default value for CommandOptions.OutputFile if not
'   specified, or appends the full path if a filename only is given.
'
'   Returns FALSE if SearchString is not specified in the commandline,
'   or TRUE if it is.
'
'---------------------------------------------------------------------------

Dim DefaultOutputFile As String

    DefaultOutputFile = AppPath & App.EXEName & ".txt"
    
    If CommandOptions.OutputFile = "" Then CommandOptions.OutputFile = DefaultOutputFile
    
    If PathExistFull(CommandOptions.OutputFile) Then 'Valid file and path name
        ValidateCommandOptions = True
        Exit Function
      Else ' Check for valid file name and append full path
        If InStr(1, CommandOptions.OutputFile, "\", vbTextCompare) = 0 Then
            If ValidFilePathName(CommandOptions.OutputFile) Then
                CommandOptions.OutputFile = AppPath & CommandOptions.OutputFile
                ValidateCommandOptions = True
                Exit Function
            End If
        End If
    End If
    
    ValidateCommandOptions = False
    
End Function

Public Sub RunSearch(ByVal SearchString As String)
'---------------------------------------------------------------------------
'{RunSearch_CommandLine} executes the search based on command-line
'   arguments supplied by the user and stored in the Options and
'   CommandOptions types.  NOTE: This routine assumes that all parameters
'   have been externally validated.
'
'---------------------------------------------------------------------------
Dim Pi As Object

    If Options.DisplayGUI = 1 Then 'Called from Search GUI, set StartTime
        Process.StartTime = Timer
    End If
    
    Set Pi = New TextPos
    
    With Pi
        .SearchString = SearchString
        .SearchFileName = Options.SearchFileName
        .SearchWindowSize = Options.SearchWindowSize
        .SearchStartPosition = Options.SearchStartPosition
        .AdjacentDigits = Options.AdjacentDigits
    End With
    
    With Result
        .StatusFlag = Pi.TextPos
        .TextPosition = SpaceParse(Pi.ResultTextPosition)
        .LeftDigits = Pi.ResultLeftDigits
        .RightDigits = Pi.ResultRightDigits
        .SearchTime = SpaceParse(Pi.ResultSearchTime)
        .FileSize = SpaceParse(Pi.ResultFileSize)
        .ErrorCode = SpaceParse(Pi.ResultErrorCode)
    End With
    
    Set Pi = Nothing
    
End Sub
Public Sub Settings(ByVal Mode As Integer)
'---------------------------------------------------------------------------
'Settings loads or writes data from or to .INI file.
'
'   Mode = 1 - Load settings from .INI file, or defaults if .INI not exist
'          2 - Save current settings to .INI file
'
'---------------------------------------------------------------------------
On Error Resume Next
Dim INIFile As String, InputString As String

'INI File Tags
Const SEARCH_FILE_NAME = "SearchFileName"
Const SEARCH_WINDOW_SIZE = "SearchWindowSize"
Const DISPLAY_GUI = "DisplayGUI"
Const SEARCH_START_POSITION = "SearchStartPosition"
Const ADJACENT_DIGITS = "AdjacentDigits"
Const BCD_MODE = "BCDMode"

Const OPT_DELIM = "="

'Default Option Values
Const DEFAULT_SEARCH_WINDOW_SIZE = 131072
Const DEFAULT_SEARCH_START_POSITION = 1
Const DEFAULT_ADJACENT_DIGITS = 20
Const DEFAULT_DISPLAY_GUI = 1

    'Set INI File Name Path
    INIFile = AppPath & App.EXEName & ".ini"
    
    Select Case Mode
        Case 1 'Load Settings from file
            Options.SearchWindowSize = DEFAULT_SEARCH_WINDOW_SIZE
            Options.SearchStartPosition = DEFAULT_SEARCH_START_POSITION
            Options.AdjacentDigits = DEFAULT_ADJACENT_DIGITS
            Options.DisplayGUI = DEFAULT_DISPLAY_GUI
            If FileExist(INIFile) Then
                Open INIFile For Input As #1
                Do
                    Input #1, InputString
                    Select Case ItemParse(InputString, 1, OPT_DELIM)
                        Case SEARCH_FILE_NAME
                            Options.SearchFileName = ItemParse(InputString, 2, OPT_DELIM)
                        Case SEARCH_WINDOW_SIZE
                            Options.SearchWindowSize = ItemParse(InputString, 2, OPT_DELIM)
                        Case SEARCH_START_POSITION
                            Options.SearchStartPosition = ItemParse(InputString, 2, OPT_DELIM)
                        Case ADJACENT_DIGITS
                            Options.AdjacentDigits = ItemParse(InputString, 2, OPT_DELIM)
                        Case DISPLAY_GUI
                            Options.DisplayGUI = 1 And CInt(Val(ItemParse(InputString, 2, OPT_DELIM)))
                        Case BCD_MODE
                            Options.BCDMode = 1 And CInt(Val(ItemParse(InputString, 2, OPT_DELIM)))
                    End Select
                Loop Until EOF(1)
              Else
                Settings (2) 'Create initial .INI file
            End If
        Case 2 'Save Settings to file
            If PathExistFull(INIFile) Then
                Open INIFile For Output As #1
                    Print #1, SEARCH_FILE_NAME & OPT_DELIM & Options.SearchFileName
                    Print #1, SEARCH_WINDOW_SIZE & OPT_DELIM & Options.SearchWindowSize
                    Print #1, SEARCH_START_POSITION & OPT_DELIM & Options.SearchStartPosition
                    Print #1, ADJACENT_DIGITS & OPT_DELIM & Options.AdjacentDigits
                    Print #1, DISPLAY_GUI & OPT_DELIM & Options.DisplayGUI
                    Print #1, BCD_MODE & OPT_DELIM & Options.BCDMode
            End If
    End Select
    
    Close #1

End Sub

Public Sub UpdateSearchGUI()
    
    frmTextPos!lblTextPosition(1) = Result.TextPosition
    If Result.ErrorCode = 0 Then
        If Result.TextPosition = 0 Then
            frmTextPos!lblMessages(1) = "Not Found"
          Else
            frmTextPos!lblMessages(1) = "Search Complete"
        End If
      Else
        frmTextPos!lblMessages(1) = "Error Code: " & Result.ErrorCode
    End If
    
    frmTextPos!lblLeftDigits(1) = Result.LeftDigits
    frmTextPos!lblRightDigits(1) = Result.RightDigits
    
    Process.ElapsedTime = Abs(Timer - Process.StartTime)
    frmTextPos!lblSearchTime(1) = Process.ElapsedTime

End Sub
